/*/////////////////////////////////////////////////////////////////
  DIGITAL PINS:
  pin 2 = Start button
  pin 3 = Mode button
  pin 5 = RWPM
  pin 6 = LWPM
  pin 7 = Upper end-switch
  pin 8 = Lower end-switch

  ANALOG PINS:
  pin A0 = Potenciometer
  pin A1 = Joystick x-axis  -  hodnoty joysticku od 20 do 1018, neutralni pozice 550
  pin A2 = Joystick y-axis
  pin A4 = SDA
  pin A5 = SCL
  ///////////////////////////////////////////////////////////////////*/

#include <Wire.h>
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2);

// Tlacitka a koncove spinace
byte Start_sw = HIGH;
byte Start_sw_old = HIGH;
byte Mode_sw = HIGH;
byte Mode_sw_old = HIGH;
byte Upper_switch = HIGH;
byte Lower_switch = HIGH;

// Promene
byte Mode = 0;
byte Mode_old = 0;
byte Start_pressed = 0;
byte Prom = 0;

// Casove promene
unsigned long Time_old = 0;
unsigned long Time_old2 = 0;

//Promene do Potenciometer()
int readIndex = 0;
int total = 0;
const int numReadings = 20; // Možno upravit podle potřeby - vyhlazuje blikání procent rychlosti
int readings[numReadings];
int Average_pot;
int Average_pot_old;

//Promene pro Joystick
int Real_joy = 0;
int Real_joy_old = 0;
int Zero_joy;


void setup() {

  // Nastaveni vstupu a vysptupu
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7, INPUT_PULLUP);
  pinMode(8, INPUT_PULLUP);
  pinMode(A0, INPUT_PULLUP);
  pinMode(A1, INPUT_PULLUP);

  // Zapnuti displeje
  lcd.init();
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Mode:automatic");
  lcd.setCursor(0, 1);
  lcd.print("Speed:");
  LCD_speed_change();
  LCD_mode_change();

  Zero_joy = analogRead(A1); // Urceni nuloveho bodu joysticku


  Serial.begin(9600);
}

/////////////////////////////////////////////////////////// HLAVNI PROGRAM ////////////////////////////////////////////////////////////////////////////////////
void loop() {

  unsigned long Time_now = millis();

  //Nacteni hodnot do promenych
  Start_sw = digitalRead(2);
  Mode_sw = digitalRead(3);
  Upper_switch = digitalRead(7);
  Lower_switch = digitalRead(8);
  Real_joy = analogRead(A1);

  Potenciometer();
  LCD_speed_change();

  // Vyhlazeni signalu tlacitka
  if (Time_now - Time_old >= 20) {

    // Stisk Mode tlacitka
    if (Mode_sw == LOW && Mode_sw_old == HIGH) {
      if (Mode < 2) Mode++;
      else Mode = 0;

      Prom = 0;  // Pri zmene modu zacinaji mody vzdy v rezimu motor_stop
      Start_pressed = 0; // Pri zmene modu zacina mode2() v rezimu motor_stop
      LCD_mode_change();
      LCD_speed_change();
      Time_old = Time_now;
    }
  }

  // Rezimy otviraku
  switch (Mode) {
    case 0:
      Mode1();
      break;
    case 1:
      Mode2();
      break;
    case 2:
      Mode3();
      break;
  }

  // Nacteni nynejsich hodnot do starych
  Start_sw_old = Start_sw;
  Mode_sw_old = Mode_sw;
  Average_pot_old = Average_pot;
  Real_joy_old = Real_joy;
  Mode_old = Mode;

  // Serial.println();
}
///////////////////////////////////////////////////// KONEC HLAVNIHO PROGRAMU //////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////// JEDNOTLIVE MODY ///////////////////////////////////////////////////////////////////////////////////////////////
void Mode1() {

  if (Upper_switch == LOW && Lower_switch == LOW) {  // Ochranný if, v normálním provozu nemůže nastat
    Prom = 0;
  }
  else {

    if (Start_sw_old == HIGH && Start_sw == LOW && Upper_switch == LOW && Lower_switch == HIGH) Prom = 1;

    if (Lower_switch == LOW) Prom = 2;

    if (Prom == 2 && Upper_switch == LOW) Prom = 0;

    if (Prom == 0 && Start_sw_old == HIGH && Start_sw == LOW && Upper_switch == HIGH) Prom = 2;

  }

  switch (Prom) {
    case 0:
      Motor_stop();
      break;
    case 1:
      Motor_down();
      break;
    case 2:
      Motor_up();
      break;
  }

}

void Mode2() {

  // Při stisknutí start tlačítka, aktualizuj displej (kvůli zobrazení On/Off)
  if (Start_sw_old == HIGH && Start_sw == LOW) {
    if (Start_pressed > 0) Start_pressed = 0;
    else Start_pressed = 1;

    LCD_mode_change();
  }

  if (Upper_switch == LOW && Lower_switch == LOW) {  // Ochranný if, v normálním provozu nemůže nastat
    Prom = 0;
  }
  else {

    if (Prom == 0 && Start_pressed == 1) Prom = 2;

    if (Upper_switch == LOW && Prom == 2 && Start_pressed == 1) Prom = 1;
    else if (Upper_switch == LOW && Prom == 2 && Start_pressed == 0) Prom = 0;

    if (Lower_switch == LOW && Prom == 1) Prom = 2;
    
  }

  switch (Prom) {
    case 0:
      Motor_stop();
      break;
    case 1:
      Motor_down();
      break;
    case 2:
      Motor_up();
      break;
  }

}

void Mode3() {

  /// Prepsani stavu zapnuti PARTY modu ///
  lcd.setCursor(13, 0);
  lcd.print("   ");


  int Motor_speed_up = 0;
  int Motor_speed_down = 0;

  Motor_speed_up = abs((Real_joy - Zero_joy) / 530.0 * 128.0);
  Motor_speed_down = abs((Real_joy - Zero_joy) / 465.0 * 128.0);

  if (Upper_switch == LOW && Lower_switch == LOW) {  // Ochranný if, v normálním provozu nemůže nastat
    Motor_stop();
  }
  else {

    /// Motor nahoru ///
    if (Motor_speed_up >= 255) Motor_speed_up = 255; // Pojisteni proti presahu maximalni hodnoty analogoveho vstupu

    if (Real_joy <= Zero_joy && Upper_switch == HIGH) {
      analogWrite(5, LOW);
      analogWrite(6, Motor_speed_up);
    } else if (Real_joy <= Zero_joy && Upper_switch == LOW) Motor_stop(); // V pripade doteku horniho koncoveho spinace nejede motor vice nahoru

    /// Motor dolu ///
    if (Motor_speed_down >= 255) Motor_speed_down = 255; // Pojisteni proti presahu maximalni hodnoty analogoveho vstupu

    if (Real_joy >= Zero_joy && Lower_switch == HIGH) {
      analogWrite(5, Motor_speed_down);
      analogWrite(6, LOW);
    } else if (Real_joy >= Zero_joy && Lower_switch == LOW) Motor_stop(); // V pripade doteku dolniho koncoveho spinace nejede motor vice dolu

  }

}
//////////////////////////////////////////////////////////////////////////// KONEC MODU /////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////// POTENCIOMETR /////////////////////////////////////////////////////////////////////
void Potenciometer() {
  /*
    Bere poslednich numReadings namerenych hodnot z potenciometru a dela z toho prumer
  */

  // Odecteni posledni hodnoty z total
  total = total - readings[readIndex];

  // Pricitani nove namerene hodnoty do total
  readings[readIndex] = analogRead(A0);
  total = total + readings[readIndex];
  readIndex++;

  if (readIndex >= numReadings) {
    readIndex = 0;
  }

  // Vypocet prumeru z promene total a prepocitani na analogovy vystup
  Average_pot = abs(((total / numReadings) - 1023) / 1023.0 * 128.0); // Uměle omezeno na 50% maximální rychlosti motoru
}


///////////////////////////////////////////////////////////////////////////////// LCD ////////////////////////////////////////////////////////////////////////
/// Aktualizace rychlosti na LCD ///
void LCD_speed_change() {

  unsigned long Time_now = millis();
  int Percentage_pot;
  int Percentage_joy_up;
  int Percentage_joy_down;


  // Vypocet procentualni rychlosti otaceni motoru - potenciometr
  Percentage_pot = abs(((total / numReadings) - 1023) / 1023.0 * 102.0); // *102 je tam proto, aby uzivatel videl hodnoty 0-100% a neukazovala se realna odchylka potenciometru

  // Vypocet procentualni rychlosti otaceni motoru - joystick
  // Hodnoty joysticku nejsou rovnomerne, z mereni vychazi minimalni hodnota 20 a maximalni 1018
  Percentage_joy_up = abs(map(Real_joy, Zero_joy, 20, 0, 100));
  Percentage_joy_down = abs(map(Real_joy, Zero_joy, 1018, 0, 100));


  // Aktualizace hodnoty potenciometru
  if (Mode == 0 || Mode == 1) {
    if (Mode != Mode_old) {  // Aktualizace pri zmene modu
      lcd.setCursor(6, 1);
      lcd.print("        ");
      lcd.setCursor(6, 1);
      lcd.print(Percentage_pot);
      lcd.print("%");
    }
    if (Average_pot != Average_pot_old) {  // Aktualizace pri otoceni potenciometrem
      lcd.setCursor(6, 1);
      lcd.print("        ");
      lcd.setCursor(6, 1);
      lcd.print(Percentage_pot);
      lcd.print("%");

    }
  }

  // Aktualizace hodnoty joysticku
  if (Mode == 2) {
    if (Mode != Mode_old) {  // Aktualizace pri zmene modu
      lcd.setCursor(6, 1);
      lcd.print("        ");
      lcd.setCursor(6, 1);
      lcd.print(Percentage_joy_up);
      lcd.print("%");
    }

    if (Time_now - Time_old2 >= 20) { // Vyhlazeni zobrazovani cisla na displeji

      if (Real_joy != Real_joy_old) {  // Aktualizace pri pohybu joystickem
        if (Real_joy <= Zero_joy && Upper_switch == HIGH && Lower_switch == HIGH) {  // Rychlost pohybu nahoru
          lcd.setCursor(6, 1);
          lcd.print("        ");
          lcd.setCursor(6, 1);
          lcd.print(Percentage_joy_up);
          lcd.print("%");
        } else if (Real_joy >= Zero_joy && Upper_switch == HIGH && Lower_switch == HIGH) {  // Rychlost pohybu dolu
          lcd.setCursor(6, 1);
          lcd.print("        ");
          lcd.setCursor(6, 1);
          lcd.print(Percentage_joy_down);
          lcd.print("%");
        } else if (Upper_switch == LOW || Lower_switch == LOW) {  // Pri doteku koncoveho spinace je rychlost 0%
          lcd.setCursor(6, 1);
          lcd.print("        ");
          lcd.setCursor(6, 1);
          lcd.print("0%");
        }
      }
      Time_old2 = Time_now;
    }
  }
}

/// Aktualizace modu na LCD ///
void LCD_mode_change() {

  if (Mode == 0) {
    lcd.setCursor(5, 0);
    lcd.print("         ");
    lcd.setCursor(5, 0);
    lcd.print("automatic");
  }

  if (Mode == 1) {
    lcd.setCursor(5, 0);
    lcd.print("         ");
    lcd.setCursor(5, 0);
    lcd.print("PARTY!!");

    if (Start_pressed == 0) {
      lcd.setCursor(13, 0);
      lcd.print("   ");
      lcd.setCursor(13, 0);
      lcd.print("Off");
    }

    if (Start_pressed == 1) {
      lcd.setCursor(13, 0);
      lcd.print("   ");
      lcd.setCursor(13, 0);
      lcd.print("On");
    }
  }

  if (Mode == 2) {
    lcd.setCursor(5, 0);
    lcd.print("         ");
    lcd.setCursor(5, 0);
    lcd.print("manual");
  }

}

/////////////////////////////////////////////////////////////////// RIZENI OTACENI MOTORU /////////////////////////////////////////////////////////////////////
void Motor_up() {
  analogWrite(5, LOW);
  analogWrite(6, Average_pot);
}

void Motor_down() {
  analogWrite(5, Average_pot);
  analogWrite(6, LOW);
}

void Motor_stop() {
  digitalWrite(5, LOW);
  digitalWrite(6, LOW);
}
